<?php

namespace Growmatik\Woocommerce;

use Growmatik\GraphQL_Client;

use WC_Order;

use function Growmatik\transfer_an_event;
use function Growmatik\get_user_identity_token;

class Order_Status_Change_Hooks {

	private $growmatik_statuses = array(
		'placed'     => 'order_placed',
		'pending'    => 'order_pending',
		'on-hold'    => 'order_on_hold',
		'processing' => 'order_processing',
		'completed'  => 'order_completed',
		'cancelled'  => 'order_cancelled',
		'failed'     => 'order_failed',
		'refunded'   => 'order_refunded',
	);

	public function __construct( $available_triggers = array() ) {
		if ( ! is_woocommerce_active() ) {
			return;
		}

		$this->available_triggers = array_unique( $available_triggers );

		add_action( 'woocommerce_order_status_changed', array( $this, 'order_changed' ), 10, 3 );

		add_action( 'user_register', array( $this, 'customer_created' ), 10, 2 );
	}

	public function order_changed( $order_id, $status_from, $status_to ) {
		if ( ! in_array( $status_to, array_keys( $this->growmatik_statuses ), true ) ) {
			return;
		}

		$order = new WC_Order( $order_id );
		if ( $order->get_user_id() <= 0 ) {
			if ( 'order_placed' === $this->growmatik_statuses[ $status_to ] ) {
				$this->reset_guest_checkout_started_cart_id( $order );
			}

			return;
		}

		$variables_types = array(
			'siteId'      => 'ID!',
			'siteUser'    => 'SiteUserInputType',
			'orderStatus' => 'String!',
			'orderData'   => 'JsonType',
		);

		$items = array_map(
			function ( $item ) {
				return $item->get_data();
			},
			$order->get_items()
		);

		foreach ( $items as &$item ) {
			$image         = wp_get_attachment_image_src(
				get_post_thumbnail_id( $item['product_id'] ),
				'single-post-thumbnail'
			);
			$item['image'] = $image ? $image[0] : '';
		}

		$order_data = array(
			'order' => $order->get_data(),
			'items' => $items,
		);

		$user_id = $order->get_user_id();
		$data    = array(
			'siteId'      => get_option( 'growmatik_site_id' ),
			'siteUser'    => array(
				'userId'        => $user_id,
				'identityToken' => get_user_identity_token( $user_id ),
			),
			'orderStatus' => $this->growmatik_statuses[ $status_to ],
			'orderData'   => wp_json_encode( $order_data ),
		);

		GraphQL_Client::make()
			->make_mutation( 'orderStatusChangeTrigger', $variables_types, 'success', $data )
			->execute();
	}

	public function reset_guest_checkout_started_cart_id( $order ) {
		$variables_types = array(
			'siteId'   => 'ID!',
			'platform' => 'PlatformEnum!',
			'email'    => 'String',
			'orderId'  => 'String',
		);
		$email           = $order->get_billing_email();
		$data            = array(
			'siteId'   => get_option( 'growmatik_site_id' ),
			'email'    => $email,
			'platform' => 'woo',
			'orderId'  => (string) $order->get_id(),
		);

		GraphQL_Client::make()
			->make_mutation( 'siteUserCheckoutStartedTriggered', $variables_types, null, $data )
			->execute();
	}

	public function customer_created( $user_id, $user_data ) {
		if ( ( $user_data['role'] ?? null ) !== 'customer' ) {
			return;
		}
		unset( $user_data['user_pass'] );
		transfer_an_event( 'wp_customer_created', $user_id, $user_data );
	}
}
